@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,29 +13,24 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import math
 import traceback
+from typing import Any
 
 from hypothesis import Verbosity, settings
 from hypothesis.errors import CleanupFailed, InvalidArgument, UnsatisfiedAssumption
-from hypothesis.internal.compat import string_types
 from hypothesis.internal.conjecture.data import ConjectureData
+from hypothesis.internal.reflection import deprecated_posargs
 from hypothesis.internal.validation import check_type
 from hypothesis.reporting import report, verbose_report
 from hypothesis.utils.dynamicvariables import DynamicVariable
 
-if False:
-    from typing import Any, AnyStr  # noqa
-
 
 def reject():
     raise UnsatisfiedAssumption()
 
 
-def assume(condition):
-    # type: (Any) -> bool
+def assume(condition: Any) -> bool:
     """Calling ``assume`` is like an :ref:`assert <python:assert>` that marks
     the example as bad, rather than failing the test.
 
@@ -55,11 +48,11 @@ _current_build_context = DynamicVariable(None)
 def current_build_context():
     context = _current_build_context.value
     if context is None:
-        raise InvalidArgument(u"No build context registered")
+        raise InvalidArgument("No build context registered")
     return context
 
 
-class BuildContext(object):
+class BuildContext:
     def __init__(self, data, is_final=False, close_on_capture=True):
         assert isinstance(data, ConjectureData)
         self.data = data
@@ -101,12 +94,11 @@ def cleanup(teardown):
     """
     context = _current_build_context.value
     if context is None:
-        raise InvalidArgument(u"Cannot register cleanup outside of build context")
+        raise InvalidArgument("Cannot register cleanup outside of build context")
     context.tasks.append(teardown)
 
 
-def note(value):
-    # type: (AnyStr) -> None
+def note(value: str) -> None:
     """Report this value in the final execution."""
     context = _current_build_context.value
     if context is None:
@@ -116,8 +108,7 @@ def note(value):
         report(value)
 
 
-def event(value):
-    # type: (AnyStr) -> None
+def event(value: str) -> None:
     """Record an event that occurred this test. Statistics on number of test
     runs with each event will be reported at the end if you run Hypothesis in
     statistics reporting mode.
@@ -131,8 +122,8 @@ def event(value):
     context.data.note_event(value)
 
 
-def target(observation, label=""):
-    # type: (float, str) -> None
+@deprecated_posargs
+def target(observation: float, *, label: str = "") -> None:
     """Calling this function with a ``float`` observation gives it feedback
     with which to guide our search for inputs that will cause an error, in
     addition to all the usual heuristics.  Observations must always be finite.
@@ -166,11 +157,16 @@ def target(observation, label=""):
         ``hypothesis.target`` is considered experimental, and may be radically
         changed or even removed in a future version.  If you find it useful,
         please let us know so we can share and build on that success!
+
+    :ref:`statistics` include the best score seen for each label,
+    which can help avoid `the threshold problem
+    <https://hypothesis.works/articles/threshold-problem/>`__ when the minimal
+    example shrinks right down to the threshold of failure (:issue:`2180`).
     """
     check_type(float, observation, "observation")
-    if math.isinf(observation) or math.isnan(observation):
+    if not math.isfinite(observation):
         raise InvalidArgument("observation=%r must be a finite float." % observation)
-    check_type(string_types, label, "label")
+    check_type(str, label, "label")
 
     context = _current_build_context.value
     if context is None:
